@@ -8,6 +8,7 @@ import sys
 from collections import defaultdict
 from datetime import datetime, timedelta, timezone
 
+import dateutil.parser
 import pandas as pd
 from dateutil import parser
 from libmozdata import versions
@@ -17,7 +18,7 @@ from bugbug import bug_snapshot, repository
 
 
 def field(bug, field):
-    if field in bug and bug[field] != "---":
+    if field in bug and bug[field] not in ("--", "---"):
         return bug[field]
 
     return None
@@ -217,7 +218,29 @@ class blocked_bugs_number(single_bug_feature):
 
 class priority(single_bug_feature):
     def __call__(self, bug, **kwargs):
-        return bug["priority"]
+        return field(bug, "priority")
+
+
+class version(single_bug_feature):
+    def __call__(self, bug, **kwargs):
+        if bug["version"] in ("Default", "Trunk", "trunk"):
+            return "Trunk"
+        elif bug["version"] in ("other", "Other Branch"):
+            return "other"
+        elif bug["version"] == "unspecified":
+            return None
+        else:
+            return "Has Value"
+
+
+class target_milestone(single_bug_feature):
+    def __call__(self, bug, **kwargs):
+        if bug["target_milestone"] == "Future":
+            return "Future"
+        elif bug["target_milestone"] == "---":
+            return None
+        else:
+            return "Has Value"
 
 
 class has_cve_in_alias(single_bug_feature):
@@ -314,6 +337,13 @@ class num_words_comments(single_bug_feature):
         return sum(len(comment["text"].split()) for comment in bug["comments"])
 
 
+class has_attachment(single_bug_feature):
+    name = "Attachment present"
+
+    def __call__(self, bug, **kwargs):
+        return len(bug["attachments"]) > 0
+
+
 class has_image_attachment_at_bug_creation(single_bug_feature):
     name = "Image attachment present at bug creation"
 
@@ -462,6 +492,70 @@ class had_severity_enhancement(single_bug_feature):
         return False
 
 
+def get_time_to_fix(bug):
+    if bug["resolution"] != "FIXED":
+        return None
+
+    if bug["cf_last_resolved"] is None:
+        return None
+
+    return (
+        dateutil.parser.parse(bug["cf_last_resolved"]).replace(tzinfo=None)
+        - dateutil.parser.parse(bug["creation_time"]).replace(tzinfo=None)
+    ).total_seconds() / 86400
+
+
+class time_to_fix(single_bug_feature):
+    def __call__(self, bug, **kwargs):
+        return get_time_to_fix(bug)
+
+
+def get_time_to_assign(bug):
+    for history in bug["history"]:
+        for change in history["changes"]:
+            if (
+                change["field_name"] == "status"
+                and change["removed"] in ("UNCONFIRMED", "NEW")
+                and change["added"] == "ASSIGNED"
+            ):
+                return (
+                    dateutil.parser.parse(history["when"]).replace(tzinfo=None)
+                    - dateutil.parser.parse(bug["creation_time"]).replace(tzinfo=None)
+                ).total_seconds() / 86400
+
+    return None
+
+
+class time_to_assign(single_bug_feature):
+    def __call__(self, bug, **kwargs):
+        return get_time_to_assign(bug)
+
+
+class cc_number(single_bug_feature):
+    def __call__(self, bug, **kwargs):
+        return len(bug["cc"])
+
+
+class is_uplifted(single_bug_feature):
+    def __call__(self, bug, **kwargs):
+        return any(
+            change["added"].startswith("approval-mozilla")
+            and change["added"].endswith("+")
+            for history in bug["history"]
+            for change in history["changes"]
+        )
+
+
+class resolution(single_bug_feature):
+    def __call__(self, bug, **kwargs):
+        return bug["resolution"]
+
+
+class status(single_bug_feature):
+    def __call__(self, bug, **kwargs):
+        return bug["status"]
+
+
 class couple_common_whiteboard_keywords(couple_bug_feature):
     def __call__(self, bugs, **kwargs):
         return [
@@ -587,6 +681,10 @@ class BugExtractor(BaseEstimator, TransformerMixin):
         self.merge_data = merge_data
 
     def fit(self, x, y=None):
+        for feature in self.feature_extractors:
+            if hasattr(feature, "fit"):
+                feature.fit(x())
+
         return self
 
     def transform(self, bugs):
@@ -642,14 +740,11 @@ class BugExtractor(BaseEstimator, TransformerMixin):
                 if res is None:
                     continue
 
-                if isinstance(res, list) or isinstance(res, set):
+                if isinstance(res, (list, set)):
                     for item in res:
-                        data[sys.intern(f"{item} in {feature_extractor_name}")] = "True"
+                        data[sys.intern(f"{item} in {feature_extractor_name}")] = True
                     continue
 
-                if isinstance(res, bool):
-                    res = str(res)
-
                 data[feature_extractor_name] = res
 
             if is_couple:
@@ -661,8 +756,6 @@ class BugExtractor(BaseEstimator, TransformerMixin):
             else:
                 reporter_experience_map[bug["creator"]] += 1
 
-                # TODO: Try simply using all possible fields instead of extracting features manually.
-
                 summary = bug["summary"]
                 comments = [c["text"] for c in bug["comments"]]
                 for cleanup_function in self.cleanup_functions:
