@@ -3,10 +3,11 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this file,
 # You can obtain one at http://mozilla.org/MPL/2.0/.
 
+import collections
 import csv
 import re
 from datetime import datetime
-from typing import Dict, Iterator, List, NewType, Optional
+from typing import Dict, Iterable, Iterator, List, NewType, Optional
 
 import tenacity
 from dateutil.relativedelta import relativedelta
@@ -21,48 +22,51 @@ BUGS_DB = "data/bugs.json"
 db.register(
     BUGS_DB,
     "https://community-tc.services.mozilla.com/api/index/v1/task/project.bugbug.data_bugs.latest/artifacts/public/bugs.json.zst",
-    6,
+    7,
 )
 
 PRODUCTS = (
-    "Add-on SDK",
-    "Android Background Services",
+    "Cloud Services",
     "Core",
     "Core Graveyard",
+    "Data Platform and Tools",
     "DevTools",
     "DevTools Graveyard",
     "External Software Affecting Firefox",
+    "Fenix",
     "Firefox",
     "Firefox Graveyard",
     "Firefox Build System",
-    "Firefox for Android",
-    "Firefox for Android Graveyard",
-    # 'Firefox for iOS',
-    "Firefox Health Report",
-    # 'Focus',
-    # 'Hello (Loop)',
+    "GeckoView",
     "Invalid Bugs",
+    "JSS",
     "NSPR",
     "NSS",
+    "Release Engineering",
+    "Remote Protocol",
+    "Shield",
+    "Testing",
     "Toolkit",
     "Toolkit Graveyard",
+    "Web Compatibility",
     "WebExtensions",
 )
 
 ATTACHMENT_INCLUDE_FIELDS = [
     "id",
-    "is_obsolete",
     "flags",
     "is_patch",
-    "creator",
     "content_type",
     "creation_time",
+    "file_name",
 ]
 
-COMMENT_INCLUDE_FIELDS = ["id", "count", "text", "author", "creation_time"]
+COMMENT_INCLUDE_FIELDS = ["id", "count", "text", "creation_time"]
 
 PRODUCT_COMPONENT_CSV_REPORT_URL = "https://bugzilla.mozilla.org/report.cgi"
 
+PHAB_REVISION_PATTERN = re.compile(r"phabricator-D([0-9]+)-url.txt")
+
 
 def get_bugs(include_invalid: Optional[bool] = False) -> Iterator[BugDict]:
     yield from (
@@ -145,35 +149,39 @@ def get(ids_or_query):
     return new_bugs
 
 
-def get_ids_between(date_from, date_to, security=False):
+def get_ids_between(date_from, date_to=None, security=False, resolution=None):
     params = {
         "f1": "creation_ts",
         "o1": "greaterthan",
         "v1": date_from.strftime("%Y-%m-%d"),
-        "f2": "creation_ts",
-        "o2": "lessthan",
-        "v2": date_to.strftime("%Y-%m-%d"),
         "product": PRODUCTS,
     }
 
+    if date_to is not None:
+        params["f2"] = "creation_ts"
+        params["o2"] = "lessthan"
+        params["v2"] = date_to.strftime("%Y-%m-%d")
+
     if not security:
         params["f3"] = "bug_group"
         params["o3"] = "isempty"
 
+    if resolution is not None:
+        params["resolution"] = resolution
+
     return get_ids(params)
 
 
-def download_bugs(bug_ids, products=None, security=False):
+def download_bugs(bug_ids: Iterable[int], security: bool = False) -> List[BugDict]:
     old_bug_count = 0
-    new_bug_ids = set(int(bug_id) for bug_id in bug_ids)
+    new_bug_ids_set = set(int(bug_id) for bug_id in bug_ids)
     for bug in get_bugs(include_invalid=True):
         old_bug_count += 1
-        if int(bug["id"]) in new_bug_ids:
-            new_bug_ids.remove(int(bug["id"]))
+        new_bug_ids_set.discard(int(bug["id"]))
 
     print(f"Loaded {old_bug_count} bugs.")
 
-    new_bug_ids = sorted(list(new_bug_ids))
+    new_bug_ids = sorted(list(new_bug_ids_set))
 
     chunks = (
         new_bug_ids[i : (i + Bugzilla.BUGZILLA_CHUNK_SIZE)]
@@ -184,17 +192,16 @@ def download_bugs(bug_ids, products=None, security=False):
         stop=tenacity.stop_after_attempt(7),
         wait=tenacity.wait_exponential(multiplier=1, min=16, max=64),
     )
-    def get_chunk(chunk):
+    def get_chunk(chunk: List[int]) -> List[BugDict]:
         new_bugs = get(chunk)
 
         if not security:
             new_bugs = [bug for bug in new_bugs.values() if len(bug["groups"]) == 0]
 
-        if products is not None:
-            new_bugs = [bug for bug in new_bugs.values() if bug["product"] in products]
-
         return new_bugs
 
+    all_new_bugs = []
+
     with tqdm(total=len(new_bug_ids)) as progress_bar:
         for chunk in chunks:
             new_bugs = get_chunk(chunk)
@@ -203,6 +210,10 @@ def download_bugs(bug_ids, products=None, security=False):
 
             db.append(BUGS_DB, new_bugs)
 
+            all_new_bugs += new_bugs
+
+    return all_new_bugs
+
 
 def _find_linked(
     bug_map: Dict[int, BugDict], bug: BugDict, link_type: str
@@ -311,3 +322,52 @@ def get_product_component_count(months: int = 12) -> Dict[str, int]:
             bugs_number[full_comp] = value
 
     return bugs_number
+
+
+def get_component_team_mapping() -> Dict[str, Dict[str, str]]:
+    r = utils.get_session("bugzilla").get(
+        "https://bugzilla.mozilla.org/rest/product?type=accessible&include_fields=name&include_fields=components.name&include_fields=components.team_name",
+        headers={"X-Bugzilla-API-Key": Bugzilla.TOKEN, "User-Agent": "bugbug"},
+    )
+    r.raise_for_status()
+
+    mapping: Dict[str, Dict[str, str]] = collections.defaultdict(dict)
+    for product in r.json()["products"]:
+        for component in product["components"]:
+            mapping[product["name"]][component["name"]] = component["team_name"]
+
+    return mapping
+
+
+def get_groups_users(group_names: List[str]) -> List[str]:
+    r = utils.get_session("bugzilla").get(
+        "https://bugzilla.mozilla.org/rest/group",
+        params={
+            "names": group_names,
+            "membership": "1",
+        },
+        headers={"X-Bugzilla-API-Key": Bugzilla.TOKEN, "User-Agent": "bugbug"},
+    )
+    r.raise_for_status()
+
+    return [
+        member["email"]
+        for group in r.json()["groups"]
+        for member in group["membership"]
+    ]
+
+
+def get_revision_ids(bug: BugDict) -> List[int]:
+    revision_ids = []
+
+    for attachment in bug["attachments"]:
+        if attachment["content_type"] != "text/x-phabricator-request":
+            continue
+
+        match = PHAB_REVISION_PATTERN.search(attachment["file_name"])
+        if match is None:
+            continue
+
+        revision_ids.append(int(match.group(1)))
+
+    return revision_ids
