@@ -3,10 +3,10 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this file,
 # You can obtain one at http://mozilla.org/MPL/2.0/.
 
+import pickle
 from collections import defaultdict
 from typing import Any, Dict, List, Tuple
 
-import joblib
 import matplotlib
 import numpy as np
 import shap
@@ -23,7 +23,7 @@ from sklearn.metrics import precision_recall_fscore_support
 from sklearn.model_selection import cross_validate, train_test_split
 from tabulate import tabulate
 
-from bugbug import bugzilla, db, repository
+from bugbug import bugzilla, db, github, repository
 from bugbug.nlp import SpacyVectorizer
 from bugbug.utils import split_tuple_generator, to_array
 
@@ -192,13 +192,7 @@ class Model:
                 feature_name = f"Comments contain '{feature_name}'"
             elif type_ == "text":
                 feature_name = f"Combined text contains '{feature_name}'"
-            elif type_ == "data":
-                if " in " in feature_name and feature_name.endswith("=True"):
-                    feature_name = feature_name[: -len("=True")]
-            elif type_ == "couple_data":
-                if " in " in feature_name and feature_name.endswith("=True"):
-                    feature_name = feature_name[: -len("=True")]
-            else:
+            elif type_ not in ("data", "couple_data"):
                 raise Exception(f"Unexpected feature type for: {full_feature_name}")
 
             cleaned_feature_names.append(feature_name)
@@ -566,16 +560,22 @@ class Model:
 
             self.clf.fit(X_train, y_train)
 
-        joblib.dump(self, self.__class__.__name__.lower())
+        with open(self.__class__.__name__.lower(), "wb") as f:
+            pickle.dump(self, f, protocol=pickle.HIGHEST_PROTOCOL)
+
         if self.store_dataset:
-            joblib.dump(X, f"{self.__class__.__name__.lower()}_data_X")
-            joblib.dump(y, f"{self.__class__.__name__.lower()}_data_y")
+            with open(f"{self.__class__.__name__.lower()}_data_X", "wb") as f:
+                pickle.dump(X, f, protocol=pickle.HIGHEST_PROTOCOL)
+
+            with open(f"{self.__class__.__name__.lower()}_data_y", "wb") as f:
+                pickle.dump(y, f, protocol=pickle.HIGHEST_PROTOCOL)
 
         return tracking_metrics
 
     @staticmethod
     def load(model_file_name: str) -> "Model":
-        return joblib.load(model_file_name)
+        with open(model_file_name, "rb") as f:
+            return pickle.load(f)
 
     def overwrite_classes(self, items, classes, probabilities):
         return classes
@@ -596,7 +596,7 @@ class Model:
         if not isinstance(items, list):
             items = [items]
 
-        assert isinstance(items[0], dict) or isinstance(items[0], tuple)
+        assert isinstance(items[0], (dict, tuple))
 
         X = self.extraction_pipeline.transform(lambda: items)
         if probabilities:
@@ -726,7 +726,7 @@ class CommitModel(Model):
 
             assert len(bug_map) > 0
 
-        for commit in repository.get_commits():
+        for commit in repository.get_commits(include_ignored=True):
             if commit["node"] not in classes:
                 continue
 
@@ -751,3 +751,17 @@ class BugCoupleModel(Model):
 
         for (bug_id1, bug_id2), label in classes.items():
             yield (bugs[bug_id1], bugs[bug_id2]), label
+
+
+class IssueModel(Model):
+    def __init__(self, lemmatization=False):
+        Model.__init__(self, lemmatization)
+        self.training_dbs = [github.GITHUB_ISSUES_DB]
+
+    def items_gen(self, classes):
+        for issue in github.get_issues():
+            issue_number = issue["number"]
+            if issue_number not in classes:
+                continue
+
+            yield issue, classes[issue_number]
