@@ -3,6 +3,9 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this file,
 # You can obtain one at http://mozilla.org/MPL/2.0/.
 
+import logging
+from typing import Dict, Iterable, List, Optional, Tuple
+
 import numpy as np
 import xgboost
 from sklearn.compose import ColumnTransformer
@@ -13,6 +16,8 @@ from sklearn.pipeline import Pipeline
 from bugbug import bug_features, bugzilla, feature_cleanup, utils
 from bugbug.model import BugModel
 
+logger = logging.getLogger(__name__)
+
 KEYWORD_DICT = {
     "sec-critical": "security",
     "sec-high": "security",
@@ -22,13 +27,66 @@ KEYWORD_DICT = {
     "sec-audit": "security",
     "sec-vector": "security",
     "sec-want": "security",
+    "csectype-bounds": "security",
+    "csectype-disclosure": "security",
+    "csectype-dos": "security",
+    "csectype-framepoisoning": "security",
+    "csectype-intoverflow": "security",
+    "csectype-jit": "security",
+    "csectype-nullptr": "security",
+    "csectype-oom": "security",
+    "csectype-other": "security",
+    "csectype-priv-escalation": "security",
+    "csectype-race": "security",
+    "csectype-sop": "security",
+    "csectype-spoof": "security",
+    "csectype-uaf": "security",
+    "csectype-undefined": "security",
+    "csectype-uninitialized": "security",
+    "csectype-wildptr": "security",
     "memory-footprint": "memory",
     "memory-leak": "memory",
     "crash": "crash",
     "crashreportid": "crash",
     "perf": "performance",
+    "topperf": "performance",
+    "power": "power",
 }
-KEYWORD_LIST = sorted(set(KEYWORD_DICT.values()))
+TYPE_LIST = sorted(set(KEYWORD_DICT.values()))
+
+
+def bug_to_types(
+    bug: bugzilla.BugDict, bug_map: Optional[Dict[int, bugzilla.BugDict]] = None
+) -> List[str]:
+    types = set()
+
+    if "[overhead" in bug["whiteboard"].lower():
+        types.add("memory")
+
+    if "[power" in bug["whiteboard"].lower():
+        types.add("power")
+
+    if "cf_crash_signature" in bug and bug["cf_crash_signature"] not in ("", "---"):
+        types.add("crash")
+
+    if bug_map is not None:
+        for bug_id in bug["blocks"]:
+            if bug_id not in bug_map:
+                continue
+
+            alias = bug_map[bug_id]["alias"]
+            if alias and alias.startswith("memshrink"):
+                types.add("memory")
+
+    return list(
+        types.union(
+            set(
+                KEYWORD_DICT[keyword]
+                for keyword in bug["keywords"]
+                if keyword in KEYWORD_DICT
+            )
+        )
+    )
 
 
 class BugTypeModel(BugModel):
@@ -96,29 +154,39 @@ class BugTypeModel(BugModel):
             xgboost.XGBClassifier(n_jobs=utils.get_physical_cpu_count())
         )
 
-    def get_labels(self):
+    def get_labels(self) -> Tuple[Dict[int, np.ndarray], List[str]]:
         classes = {}
 
-        for bug_data in bugzilla.get_bugs():
-            target = np.zeros(len(KEYWORD_LIST))
-            for keyword in bug_data["keywords"]:
-                if keyword in KEYWORD_DICT:
-                    target[KEYWORD_LIST.index(KEYWORD_DICT[keyword])] = 1
+        bug_map = {bug["id"]: bug for bug in bugzilla.get_bugs()}
+
+        for bug_data in bug_map.values():
+            target = np.zeros(len(TYPE_LIST))
+            for type_ in bug_to_types(bug_data, bug_map):
+                target[TYPE_LIST.index(type_)] = 1
 
             classes[int(bug_data["id"])] = target
 
-        return classes, KEYWORD_LIST
+        for type_ in TYPE_LIST:
+            logger.info(
+                f"{sum(1 for target in classes.values() if target[TYPE_LIST.index(type_)] == 1)} {type_} bugs"
+            )
+
+        return classes, TYPE_LIST
 
     def get_feature_names(self):
         return self.extraction_pipeline.named_steps["union"].get_feature_names()
 
-    def overwrite_classes(self, bugs, classes, probabilities):
+    def overwrite_classes(
+        self,
+        bugs: Iterable[bugzilla.BugDict],
+        classes: Dict[int, np.ndarray],
+        probabilities: bool,
+    ):
         for i, bug in enumerate(bugs):
-            for keyword in bug["keywords"]:
-                if keyword in KEYWORD_LIST:
-                    if probabilities:
-                        classes[i][KEYWORD_LIST.index(keyword)] = 1.0
-                    else:
-                        classes[i][KEYWORD_LIST.index(keyword)] = 1
+            for type_ in bug_to_types(bug):
+                if probabilities:
+                    classes[i][TYPE_LIST.index(type_)] = 1.0
+                else:
+                    classes[i][TYPE_LIST.index(type_)] = 1
 
         return classes
