@@ -3,6 +3,8 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this file,
 # You can obtain one at http://mozilla.org/MPL/2.0/.
 
+from typing import Any, Dict, List, Tuple
+
 from bugbug.models.defect import DefectModel
 
 
@@ -12,7 +14,7 @@ class DefectEnhancementTaskModel(DefectModel):
 
         self.calculate_importance = False
 
-    def get_labels(self):
+    def get_labels(self) -> Tuple[Dict[int, Any], List[Any]]:
         classes = self.get_bugbug_labels("defect_enhancement_task")
 
         print(
