@@ -39,20 +39,34 @@ db.register(
     3,
 )
 
-EVALUATION_MONTHS = 2
+EVALUATION_MONTHS = 3
 
 
 class RegressorModel(CommitModel):
     def __init__(
-        self, lemmatization: bool = False, interpretable: bool = False
+        self,
+        lemmatization: bool = False,
+        interpretable: bool = True,
+        use_finder: bool = False,
+        exclude_finder: bool = True,
+        finder_regressions_only: bool = False,
     ) -> None:
         CommitModel.__init__(self, lemmatization)
 
         self.training_dbs += [BUG_INTRODUCING_COMMITS_DB, bugzilla.BUGS_DB]
+        if finder_regressions_only:
+            self.training_dbs.append(BUG_FIXING_COMMITS_DB)
 
         self.store_dataset = True
         self.sampler = RandomUnderSampler(random_state=0)
 
+        self.use_finder = use_finder
+        self.exclude_finder = exclude_finder
+        assert (
+            use_finder ^ exclude_finder
+        ), "Using both use_finder and exclude_finder option does not make a lot of sense"
+        self.finder_regressions_only = finder_regressions_only
+
         feature_extractors = [
             commit_features.source_code_file_size(),
             commit_features.other_file_size(),
@@ -114,17 +128,33 @@ class RegressorModel(CommitModel):
     def get_labels(self):
         classes = {}
 
-        regressors = set(
-            r["bug_introducing_rev"]
-            for r in db.read(BUG_INTRODUCING_COMMITS_DB)
-            if r["bug_introducing_rev"]
+        if self.use_finder or self.exclude_finder:
+            if self.finder_regressions_only:
+                regression_fixes = set(
+                    bug_fixing_commit["rev"]
+                    for bug_fixing_commit in db.read(BUG_FIXING_COMMITS_DB)
+                    if bug_fixing_commit["type"] == "r"
+                )
+
+            regressors = set(
+                r["bug_introducing_rev"]
+                for r in db.read(BUG_INTRODUCING_COMMITS_DB)
+                if r["bug_introducing_rev"]
+                and (
+                    not self.finder_regressions_only
+                    or r["bug_fixing_rev"] in regression_fixes
+                )
+            )
+
+        regressor_bugs = set(
+            sum((bug["regressed_by"] for bug in bugzilla.get_bugs()), [])
         )
 
         for commit_data in repository.get_commits():
             if commit_data["backedoutby"]:
                 continue
 
-            if commit_data["author_email"] == "wptsync@mozilla.com":
+            if repository.is_wptsync(commit_data):
                 continue
 
             push_date = dateutil.parser.parse(commit_data["pushdate"])
@@ -134,15 +164,26 @@ class RegressorModel(CommitModel):
                 continue
 
             node = commit_data["node"]
-            if node in regressors:
+            if commit_data["bug_id"] in regressor_bugs or (
+                self.use_finder and node in regressors
+            ):
                 classes[node] = 1
-            else:
-                # The labels we have are only from two years and six months ago (see the regressor finder script).
-                if push_date < datetime.utcnow() - relativedelta(years=2, months=6):
+            elif not self.exclude_finder or node not in regressors:
+                # The labels we have are only from two years ago (see https://groups.google.com/g/mozilla.dev.platform/c/SjjW6_O-FqM/m/G-CrIVT2BAAJ).
+                # While we can go further back with the regressor finder script, it isn't remotely
+                # as precise as the "Regressed By" data.
+                # In the future, we might want to re-evaluate this limit (e.g. extend ), but we
+                # have to be careful (using too old patches might cause worse results as patch
+                # characteristics evolve over time).
+                if push_date < datetime.utcnow() - relativedelta(years=2):
                     continue
 
-                # We remove the last 6 months, as there could be regressions which haven't been filed yet.
-                if push_date > datetime.utcnow() - relativedelta(months=6):
+                # We remove the last 3 months, as there could be regressions which haven't been
+                # filed yet. While it is true that some regressions might not be found for a long
+                # time, more than 3 months seems overly conservative.
+                # There will be some patches we currently add to the clean set and will later move
+                # to the regressor set, but they are a very small subset.
+                if push_date > datetime.utcnow() - relativedelta(months=3):
                     continue
 
                 classes[node] = 0
@@ -172,7 +213,7 @@ class RegressorModel(CommitModel):
             if commit_data["backedoutby"]:
                 continue
 
-            if commit_data["author_email"] == "wptsync@mozilla.com":
+            if repository.is_wptsync(commit_data):
                 continue
 
             push_date = dateutil.parser.parse(commit_data["pushdate"])
@@ -208,7 +249,7 @@ class RegressorModel(CommitModel):
 
         print(f"Average risk is {average_regression_rate}")
 
-        MIN_SAMPLE = 300
+        MIN_SAMPLE = 200
 
         # Step 2. Define risk band 1 (half than average risk).
         max_band1_prob = 1.0
@@ -267,7 +308,7 @@ class RegressorModel(CommitModel):
         # Step 4. Define risk band 2 (average risk).
         results.sort(key=lambda x: x[0])
         for prob_start in np.arange(max_band1_prob / 2, max_band1_prob + 0.02, 0.01):
-            for prob_end in np.arange(min_band3_prob, 0.99, 0.01):
+            for prob_end in np.arange(min_band3_prob - 0.02, 0.99, 0.01):
                 total_landings = 0
                 total_regressions = 0
                 for prob, is_reg in results:
@@ -294,3 +335,10 @@ class RegressorModel(CommitModel):
 
     def get_feature_names(self):
         return self.extraction_pipeline.named_steps["union"].get_feature_names()
+
+    def overwrite_classes(self, commits, classes, probabilities):
+        for i, commit in enumerate(commits):
+            if repository.is_wptsync(commit):
+                classes[i] = 0 if not probabilities else [1.0, 0.0]
+
+        return classes
