@@ -285,7 +285,7 @@ class TestSelectModel(Model):
             (("android-hw", "debug"), 17),
             (("tsan", "opt"), 18),
             (("tsan", "debug"), 19),
-            (("test-linux1804-64-shippable/opt-*-e10s",), 1),
+            (("test-linux1804-64/opt-*-e10s",), 1),
         ]
 
         for substrings, cost in reversed(costs):
@@ -498,7 +498,10 @@ class TestSelectModel(Model):
             return set(tasks)
 
     def select_configs(
-        self, groups: Collection[str], min_redundancy_confidence: float
+        self,
+        groups: Collection[str],
+        min_redundancy_confidence: float,
+        max_configurations: int = 3,
     ) -> Dict[str, List[str]]:
         failing_together = test_scheduling.get_failing_together_db("config_group", True)
 
@@ -510,6 +513,7 @@ class TestSelectModel(Model):
             "select_configs", pywraplp.Solver.CBC_MIXED_INTEGER_PROGRAMMING
         )
 
+        config_vars = {config: solver.BoolVar(config) for config in all_configs}
         config_group_vars = {
             (config, group): solver.BoolVar(f"{group}@{config}")
             for group in groups
@@ -535,18 +539,53 @@ class TestSelectModel(Model):
 
                 seen |= equivalence_set
 
-            for equivalence_set in equivalence_sets[group]:
+            set_variables = [
+                solver.BoolVar(f"{group}_{j}")
+                for j in range(len(equivalence_sets[group]))
+            ]
+
+            for j, equivalence_set in enumerate(equivalence_sets[group]):
+                set_variable = set_variables[j]
+
                 sum_constraint = sum(
                     config_group_vars[(config, group)] for config in equivalence_set
                 )
                 if mutually_exclusive:
-                    solver.Add(sum_constraint == 1)
+                    solver.Add(sum_constraint == set_variable)
                 else:
-                    solver.Add(sum_constraint >= 1)
+                    solver.Add(sum_constraint >= set_variable)
+
+            # Cap to max_configurations equivalence sets.
+            solver.Add(
+                sum(set_variables)
+                >= (
+                    max_configurations
+                    if len(set_variables) >= max_configurations
+                    else len(set_variables)
+                )
+            )
+
+        for config in all_configs:
+            solver.Add(
+                sum(
+                    config_group_var
+                    for (c, g), config_group_var in config_group_vars.items()
+                    if config == c
+                )
+                <= config_vars[config] * len(groups)
+            )
 
         # Choose the best set of tasks that satisfy the constraints with the lowest cost.
+        # The cost is calculated as a sum of the following:
+        # - a fixed cost to use a config (since selecting a config has overhead, it is
+        #   wasteful to select a config only to run a single group);
+        # - a cost for each selected group.
+        # This way, for example, if we have a group that must run on a costly config and a
+        # group that can run either on the costly one or on a cheaper one, they'd both run
+        # on the costly one (since we have to pay its setup cost anyway).
         solver.Minimize(
-            sum(
+            sum(10 * config_costs[c] * config_vars[c] for c in config_vars.keys())
+            + sum(
                 config_costs[config] * config_group_vars[(config, group)]
                 for config, group in config_group_vars.keys()
             )
@@ -561,7 +600,7 @@ class TestSelectModel(Model):
                 if config_group_var.solution_value() == 1:
                     configs_by_group[group].append(config)
         else:
-            least_cost_config = min(config_costs, key=config_costs.get)
+            least_cost_config = min(config_costs, key=lambda c: config_costs[c])
             for group in groups:
                 configs_by_group[group].append(least_cost_config)
 
@@ -603,6 +642,30 @@ class TestSelectModel(Model):
 
         test_pushes = {push["revs"][0]: push for push in test_pushes_list}
 
+        if self.granularity == "group":
+            for (
+                revisions,
+                fix_revision,
+                push_runnables,
+                possible_regressions,
+                likely_regressions,
+            ) in tqdm(push_data_iter(), total=push_data_count):
+                if revisions[0] not in test_pushes:
+                    continue
+
+                test_pushes[revisions[0]]["config_group_failures"] = (
+                    possible_regressions + likely_regressions
+                )
+
+            missing_config_group_failures = sum(
+                1
+                for push in test_pushes.values()
+                if "config_group_failures" not in push
+            )
+            print(
+                f"{missing_config_group_failures} pushes without config_group failures"
+            )
+
         print(
             f"Testing on {len(test_pushes)} ({test_pushes_failures} with failures) out of {len(pushes)}. {len(all_tasks)} schedulable tasks."
         )
@@ -669,10 +732,37 @@ class TestSelectModel(Model):
                         f.cancel()
 
                 push = futures[future]
-                selected, number_configs = future.result()
+                selected, group_configs = future.result()
 
                 if reduction is not None and self.granularity == "group":
-                    push["number_configs"] = number_configs
+                    push["number_configs"] = len(
+                        set(
+                            sum(
+                                group_configs.values(),
+                                [],
+                            )
+                        )
+                    )
+                    selected_config_groups = set(
+                        (config, group)
+                        for group, configs in group_configs.items()
+                        for config in configs
+                    )
+                    if "config_group_failures" in push:
+                        caught_config_groups = selected_config_groups & set(
+                            push["config_group_failures"]
+                        )
+                        push["caught_one_config_group"] = (
+                            len(caught_config_groups) > 0
+                            if len(push["config_group_failures"]) != 0
+                            else None
+                        )
+                        push["caught_percentage_config_group"] = (
+                            len(caught_config_groups)
+                            / len(push["config_group_failures"])
+                            if len(push["config_group_failures"]) != 0
+                            else None
+                        )
 
                 caught = selected & set(push["failures"])
 
@@ -732,7 +822,36 @@ class TestSelectModel(Model):
                 average_configs = statistics.mean(
                     result["number_configs"] for result in test_pushes.values()
                 )
-                message += f" On average, we selected {average_configs} configs."
+                median_configs = statistics.median(
+                    result["number_configs"] for result in test_pushes.values()
+                )
+                message += f" On average, we selected {average_configs} configs (a median of {median_configs} configs)."
+
+                num_failing_pushes_with_config_group = sum(
+                    1
+                    for result in test_pushes.values()
+                    if "caught_one_config_group" in result
+                    and result["caught_one_config_group"] is not None
+                )
+                num_caught_one_config_group = sum(
+                    1
+                    for result in test_pushes.values()
+                    if "caught_one_config_group" in result
+                    and result["caught_one_config_group"]
+                )
+                percentage_caught_one_config_group = (
+                    100
+                    * num_caught_one_config_group
+                    / num_failing_pushes_with_config_group
+                )
+                average_caught_percentage_config_group = 100 * statistics.mean(
+                    result["caught_percentage_config_group"]
+                    for result in test_pushes.values()
+                    if "caught_percentage_config_group" in result
+                    and result["caught_percentage_config_group"] is not None
+                )
+
+                message += f" In {percentage_caught_one_config_group}% of pushes we caught at least one config/group failure. On average, we caught {average_caught_percentage_config_group}% of all seen config/group failures."
 
             print(message)
 
@@ -775,7 +894,7 @@ class TestConfigGroupSelectModel(TestSelectModel):
 
 
 def eval_apply_transforms(model, push, confidence_threshold, reduction, cap, minimum):
-    number_configs = None
+    group_configs = None
 
     selected = set(
         name
@@ -787,14 +906,7 @@ def eval_apply_transforms(model, push, confidence_threshold, reduction, cap, min
         if model.granularity == "label":
             selected = model.reduce(selected, reduction)
         elif model.granularity == "group":
-            number_configs = len(
-                set(
-                    sum(
-                        model.select_configs(selected, reduction).values(),
-                        [],
-                    )
-                )
-            )
+            group_configs = model.select_configs(selected, reduction)
 
     if minimum is not None and len(selected) < minimum:
         remaining = [
@@ -822,4 +934,4 @@ def eval_apply_transforms(model, push, confidence_threshold, reduction, cap, min
             )[:cap]
         )
 
-    return selected, number_configs
+    return selected, group_configs
