@@ -56,7 +56,7 @@ COMMIT_EXPERIENCES_DB = "commit_experiences.lmdb.tar.zst"
 db.register(
     COMMITS_DB,
     "https://community-tc.services.mozilla.com/api/index/v1/task/project.bugbug.data_commits.latest/artifacts/public/commits.json.zst",
-    16,
+    22,
     [COMMIT_EXPERIENCES_DB],
 )
 
@@ -127,6 +127,43 @@ def get_type(path: str) -> str:
     return EXT_TO_TYPES.get(ext, ext)
 
 
+METRIC_NAMES = [
+    "cyclomatic",
+    "halstead_N1",
+    "halstead_n1",
+    "halstead_N2",
+    "halstead_n2",
+    "sloc",
+    "ploc",
+    "lloc",
+    "cloc",
+    "blank",
+    "nargs",
+    "nexits",
+    "cognitive",
+    "mi_original",
+    "mi_sei",
+    "mi_visual_studio",
+]
+
+
+def get_total_metrics_dict() -> dict:
+    return {f"{metric}_total": 0 for metric in METRIC_NAMES}
+
+
+def get_metrics_dict() -> dict:
+    metrics = get_total_metrics_dict()
+    for metric in METRIC_NAMES:
+        metrics.update(
+            {
+                f"{metric}_avg": 0.0,
+                f"{metric}_max": 0,
+                f"{metric}_min": sys.maxsize,
+            }
+        )
+    return metrics
+
+
 class Commit:
     def __init__(
         self,
@@ -158,7 +195,7 @@ class Commit:
         self.other_deleted = 0
         self.test_deleted = 0
         self.types: Set[str] = set()
-        self.functions: Dict[str, List[Tuple[str, int, int]]] = {}
+        self.functions: Dict[str, List[dict]] = {}
         self.seniority_author = 0.0
         self.total_source_code_file_size = 0
         self.average_source_code_file_size = 0.0
@@ -175,50 +212,8 @@ class Commit:
         self.maximum_test_file_size = 0
         self.minimum_test_file_size = 0
         self.test_files_modified_num = 0
-        self.average_cyclomatic = 0.0
-        self.average_halstead_N1 = 0.0
-        self.average_halstead_n1 = 0.0
-        self.average_halstead_N2 = 0.0
-        self.average_halstead_n2 = 0.0
-        self.average_source_loc = 0.0
-        self.average_instruction_loc = 0.0
-        self.average_logical_loc = 0.0
-        self.average_comment_loc = 0.0
-        self.average_nargs = 0.0
-        self.average_nexits = 0.0
-        self.maximum_cyclomatic = 0
-        self.maximum_halstead_N2 = 0
-        self.maximum_halstead_n2 = 0
-        self.maximum_halstead_N1 = 0
-        self.maximum_halstead_n1 = 0
-        self.maximum_source_loc = 0
-        self.maximum_instruction_loc = 0
-        self.maximum_logical_loc = 0
-        self.maximum_comment_loc = 0
-        self.maximum_nargs = 0
-        self.maximum_nexits = 0
-        self.minimum_cyclomatic = sys.maxsize
-        self.minimum_halstead_N1 = sys.maxsize
-        self.minimum_halstead_n1 = sys.maxsize
-        self.minimum_halstead_N2 = sys.maxsize
-        self.minimum_halstead_n2 = sys.maxsize
-        self.minimum_source_loc = sys.maxsize
-        self.minimum_instruction_loc = sys.maxsize
-        self.minimum_logical_loc = sys.maxsize
-        self.minimum_comment_loc = sys.maxsize
-        self.minimum_nargs = sys.maxsize
-        self.minimum_nexits = sys.maxsize
-        self.total_cyclomatic = 0
-        self.total_halstead_N1 = 0
-        self.total_halstead_n1 = 0
-        self.total_halstead_N2 = 0
-        self.total_halstead_n2 = 0
-        self.total_source_loc = 0
-        self.total_instruction_loc = 0
-        self.total_logical_loc = 0
-        self.total_comment_loc = 0
-        self.total_nargs = 0
-        self.total_nexits = 0
+        self.metrics = get_metrics_dict()
+        self.metrics_diff = get_total_metrics_dict()
 
     def __eq__(self, other):
         assert isinstance(other, Commit)
@@ -278,7 +273,7 @@ def get_directories(files):
 
 
 def is_wptsync(commit: dict) -> bool:
-    return commit["author_email"] == "wptsync@mozilla.com" or any(
+    return "wptsync" in commit["author_email"] or any(
         s in commit["desc"] for s in ("wpt-pr:", "wpt-head:", "wpt-type:")
     )
 
@@ -390,13 +385,7 @@ def get_functions_from_metrics(metrics_space):
         and metrics_space["name"]
         and metrics_space["name"] != "<anonymous>"
     ):
-        functions.append(
-            {
-                "end_line": metrics_space["end_line"],
-                "name": metrics_space["name"],
-                "start_line": metrics_space["start_line"],
-            }
-        )
+        functions.append(metrics_space)
 
     for space in metrics_space["spaces"]:
         functions += get_functions_from_metrics(space)
@@ -404,9 +393,10 @@ def get_functions_from_metrics(metrics_space):
     return functions
 
 
-def get_touched_functions(metrics_space, deleted_lines, added_lines):
-    touched_functions = set()
-    touched_function_names = set()
+def get_touched_functions(
+    metrics_space: dict, deleted_lines: Iterable[int], added_lines: Iterable[int]
+) -> List[dict]:
+    touched_functions_indexes = set()
 
     functions = get_functions_from_metrics(metrics_space)
 
@@ -420,7 +410,7 @@ def get_touched_functions(metrics_space, deleted_lines, added_lines):
 
                 # If the line belongs to this function, add the function to the set of touched functions.
                 elif function["start_line"] <= line:
-                    touched_function_names.add(function["name"])
+                    touched_functions_indexes.add(functions.index(function))
                     last_f += 1
 
     # Get functions touched by added lines.
@@ -439,7 +429,7 @@ def get_touched_functions(metrics_space, deleted_lines, added_lines):
 
     for line in deleted_lines:
         for func in prev_functions:
-            if line < func["start_line"]:
+            if line <= func["start_line"]:
                 func["start_line"] += 1
 
             if line < func["end_line"]:
@@ -449,104 +439,170 @@ def get_touched_functions(metrics_space, deleted_lines, added_lines):
     get_touched(prev_functions, deleted_lines)
 
     # Return touched functions, with their new positions.
-    for function in functions:
-        if function["name"] in touched_function_names:
-            touched_functions.add(
-                (function["name"], function["start_line"], function["end_line"])
+    return [functions[i] for i in touched_functions_indexes]
+
+
+class AnalysisException(Exception):
+    """Raised when rust-code-analysis failed to analyze a file."""
+
+    pass
+
+
+def get_summary_metrics(obj, metrics_space):
+    if metrics_space["kind"] in {"unit", "function"} and metrics_space["name"] == "":
+        raise AnalysisException("Analysis error")
+
+    if metrics_space["kind"] == "function":
+        metrics = metrics_space["metrics"]
+
+        obj["cyclomatic_max"] = max(obj["cyclomatic_max"], metrics["cyclomatic"]["sum"])
+        obj["halstead_n2_max"] = max(obj["halstead_n2_max"], metrics["halstead"]["n2"])
+        obj["halstead_N2_max"] = max(obj["halstead_N2_max"], metrics["halstead"]["N2"])
+        obj["halstead_n1_max"] = max(obj["halstead_n1_max"], metrics["halstead"]["n1"])
+        obj["halstead_N1_max"] = max(obj["halstead_N1_max"], metrics["halstead"]["N1"])
+        obj["sloc_max"] = max(obj["sloc_max"], metrics["loc"]["sloc"])
+        obj["ploc_max"] = max(obj["ploc_max"], metrics["loc"]["ploc"])
+        obj["lloc_max"] = max(obj["lloc_max"], metrics["loc"]["lloc"])
+        obj["cloc_max"] = max(obj["cloc_max"], metrics["loc"]["cloc"])
+        obj["blank_max"] = max(obj["blank_max"], metrics["loc"]["blank"])
+        obj["nargs_max"] = max(obj["nargs_max"], metrics["nargs"]["sum"])
+        obj["nexits_max"] = max(obj["nexits_max"], metrics["nexits"]["sum"])
+        obj["cognitive_max"] = max(obj["cognitive_max"], metrics["cognitive"]["sum"])
+        # TODO: Try removing the None checks after https://github.com/mozilla/rust-code-analysis/issues/528 is fixed.
+        if metrics["mi"]["mi_original"] is not None:
+            obj["mi_original_max"] = max(
+                obj["mi_original_max"], metrics["mi"]["mi_original"]
+            )
+        if metrics["mi"]["mi_sei"] is not None:
+            obj["mi_sei_max"] = max(obj["mi_sei_max"], metrics["mi"]["mi_sei"])
+        if metrics["mi"]["mi_visual_studio"] is not None:
+            obj["mi_visual_studio_max"] = max(
+                obj["mi_visual_studio_max"], metrics["mi"]["mi_visual_studio"]
             )
 
-    return touched_functions
-
-
-def get_metrics(commit, metrics_space):
-    name = metrics_space["name"]
-    error = metrics_space["kind"] in {"unit", "function"} and name == ""
-
-    if metrics_space["kind"] == "function" and not error:
-        metrics = metrics_space["metrics"]
-        commit.total_cyclomatic += metrics["cyclomatic"]["sum"]
-        commit.total_halstead_n2 += metrics["halstead"]["n2"]
-        commit.total_halstead_N2 += metrics["halstead"]["N2"]
-        commit.total_halstead_n1 += metrics["halstead"]["n1"]
-        commit.total_halstead_N1 += metrics["halstead"]["N1"]
-        commit.total_source_loc += metrics["loc"]["sloc"]
-        commit.total_instruction_loc += metrics["loc"]["ploc"]
-        commit.total_logical_loc += metrics["loc"]["lloc"]
-        commit.total_comment_loc += metrics["loc"]["cloc"]
-        commit.total_nargs += metrics["nargs"]
-        commit.total_nexits += metrics["nexits"]
-
-        commit.maximum_cyclomatic = max(
-            commit.maximum_cyclomatic, metrics["cyclomatic"]["sum"]
-        )
-        commit.maximum_halstead_n2 = max(
-            commit.maximum_halstead_n2,
-            metrics["halstead"]["n2"],
-        )
-        commit.maximum_halstead_N2 = max(
-            metrics["halstead"]["N2"], commit.maximum_halstead_N2
-        )
-        commit.maximum_halstead_n1 = max(
-            metrics["halstead"]["n1"],
-            commit.maximum_halstead_n1,
-        )
-        commit.maximum_halstead_N1 = max(
-            metrics["halstead"]["N1"], commit.maximum_halstead_N1
-        )
-        commit.maximum_source_loc = max(
-            metrics["loc"]["sloc"], commit.maximum_source_loc
-        )
-        commit.maximum_instruction_loc = max(
-            metrics["loc"]["ploc"], commit.maximum_instruction_loc
-        )
-        commit.maximum_logical_loc = max(
-            metrics["loc"]["lloc"], commit.maximum_logical_loc
-        )
-        commit.maximum_comment_loc = max(
-            metrics["loc"]["cloc"], commit.maximum_comment_loc
-        )
-        commit.maximum_nargs = max(metrics["nargs"], commit.maximum_nargs)
-        commit.maximum_nexits = max(metrics["nexits"], commit.maximum_nexits)
-
-        commit.minimum_cyclomatic = min(
-            commit.minimum_cyclomatic, metrics["cyclomatic"]["sum"]
-        )
-        commit.minimum_halstead_n2 = min(
-            commit.minimum_halstead_n2,
-            metrics["halstead"]["n2"],
-        )
-        commit.minimum_halstead_N2 = min(
-            metrics["halstead"]["N2"], commit.minimum_halstead_N2
-        )
-        commit.minimum_halstead_n1 = min(
-            metrics["halstead"]["n1"],
-            commit.minimum_halstead_n1,
-        )
-        commit.minimum_halstead_N1 = min(
-            metrics["halstead"]["N1"], commit.minimum_halstead_N1
-        )
-        commit.minimum_source_loc = min(
-            metrics["loc"]["sloc"], commit.minimum_source_loc
-        )
-        commit.minimum_instruction_loc = min(
-            metrics["loc"]["ploc"], commit.minimum_instruction_loc
-        )
-        commit.minimum_logical_loc = min(
-            metrics["loc"]["lloc"], commit.minimum_logical_loc
-        )
-        commit.minimum_comment_loc = min(
-            metrics["loc"]["cloc"], commit.minimum_comment_loc
-        )
-        commit.minimum_nargs = min(metrics["nargs"], commit.minimum_nargs)
-        commit.minimum_nexits = min(metrics["nexits"], commit.minimum_nexits)
+        obj["cyclomatic_min"] = min(obj["cyclomatic_min"], metrics["cyclomatic"]["sum"])
+        obj["halstead_n2_min"] = min(obj["halstead_n2_min"], metrics["halstead"]["n2"])
+        obj["halstead_N2_min"] = min(obj["halstead_N2_min"], metrics["halstead"]["N2"])
+        obj["halstead_n1_min"] = min(obj["halstead_n1_min"], metrics["halstead"]["n1"])
+        obj["halstead_N1_min"] = min(obj["halstead_N1_min"], metrics["halstead"]["N1"])
+        obj["sloc_min"] = min(obj["sloc_min"], metrics["loc"]["sloc"])
+        obj["ploc_min"] = min(obj["ploc_min"], metrics["loc"]["ploc"])
+        obj["lloc_min"] = min(obj["lloc_min"], metrics["loc"]["lloc"])
+        obj["cloc_min"] = min(obj["cloc_min"], metrics["loc"]["cloc"])
+        obj["blank_min"] = min(obj["blank_min"], metrics["loc"]["blank"])
+        obj["nargs_min"] = min(obj["nargs_min"], metrics["nargs"]["sum"])
+        obj["nexits_min"] = min(obj["nexits_min"], metrics["nexits"]["sum"])
+        obj["cognitive_min"] = min(obj["cognitive_min"], metrics["cognitive"]["sum"])
+        # TODO: Try removing the None checks after https://github.com/mozilla/rust-code-analysis/issues/528 is fixed.
+        if metrics["mi"]["mi_original"] is not None:
+            obj["mi_original_min"] = min(
+                obj["mi_original_min"], metrics["mi"]["mi_original"]
+            )
+        if metrics["mi"]["mi_sei"] is not None:
+            obj["mi_sei_min"] = min(obj["mi_sei_min"], metrics["mi"]["mi_sei"])
+        if metrics["mi"]["mi_visual_studio"] is not None:
+            obj["mi_visual_studio_min"] = min(
+                obj["mi_visual_studio_min"], metrics["mi"]["mi_visual_studio"]
+            )
 
     for space in metrics_space["spaces"]:
-        error |= get_metrics(commit, space)
+        get_summary_metrics(obj, space)
 
-    return error
+    return obj
 
 
-def transform(hg: hglib.client, repo_dir: str, commit: Commit):
+def get_space_metrics(
+    obj: dict, metrics_space: dict, calc_summaries: bool = True
+) -> None:
+    if metrics_space["kind"] in {"unit", "function"} and metrics_space["name"] == "":
+        raise AnalysisException("Analysis error")
+
+    metrics = metrics_space["metrics"]
+    obj["cyclomatic_total"] += metrics["cyclomatic"]["sum"]
+    obj["halstead_n2_total"] += metrics["halstead"]["n2"]
+    obj["halstead_N2_total"] += metrics["halstead"]["N2"]
+    obj["halstead_n1_total"] += metrics["halstead"]["n1"]
+    obj["halstead_N1_total"] += metrics["halstead"]["N1"]
+    obj["sloc_total"] += metrics["loc"]["sloc"]
+    obj["ploc_total"] += metrics["loc"]["ploc"]
+    obj["lloc_total"] += metrics["loc"]["lloc"]
+    obj["cloc_total"] += metrics["loc"]["cloc"]
+    obj["blank_total"] += metrics["loc"]["blank"]
+    obj["nargs_total"] += metrics["nargs"]["sum"]
+    obj["nexits_total"] += metrics["nexits"]["sum"]
+    obj["cognitive_total"] += metrics["cognitive"]["sum"]
+    # TODO: Try removing the None checks after https://github.com/mozilla/rust-code-analysis/issues/528 is fixed.
+    if metrics["mi"]["mi_original"] is not None:
+        obj["mi_original_total"] += metrics["mi"]["mi_original"]
+    if metrics["mi"]["mi_sei"] is not None:
+        obj["mi_sei_total"] += metrics["mi"]["mi_sei"]
+    if metrics["mi"]["mi_visual_studio"] is not None:
+        obj["mi_visual_studio_total"] += metrics["mi"]["mi_visual_studio"]
+
+    if calc_summaries:
+        for space in metrics_space["spaces"]:
+            get_summary_metrics(obj, space)
+
+
+def set_commit_metrics(
+    commit: Commit,
+    path: str,
+    deleted_lines: List[int],
+    added_lines: List[int],
+    before_metrics: dict,
+    after_metrics: dict,
+) -> None:
+    try:
+        get_space_metrics(commit.metrics, after_metrics["spaces"])
+    except AnalysisException:
+        logger.debug(f"rust-code-analysis error on commit {commit.node}, path {path}")
+
+    before_metrics_dict = get_total_metrics_dict()
+    try:
+        if before_metrics.get("spaces"):
+            get_space_metrics(
+                before_metrics_dict, before_metrics["spaces"], calc_summaries=False
+            )
+    except AnalysisException:
+        logger.debug(f"rust-code-analysis error on commit {commit.node}, path {path}")
+
+    commit.metrics_diff = {
+        f"{metric}_total": commit.metrics[f"{metric}_total"]
+        - before_metrics_dict[f"{metric}_total"]
+        for metric in METRIC_NAMES
+    }
+
+    touched_functions = get_touched_functions(
+        after_metrics["spaces"],
+        deleted_lines,
+        added_lines,
+    )
+    if len(touched_functions) == 0:
+        return
+
+    commit.functions[path] = []
+
+    for func in touched_functions:
+        metrics_dict = get_total_metrics_dict()
+
+        try:
+            get_space_metrics(metrics_dict, func, calc_summaries=False)
+        except AnalysisException:
+            logger.debug(
+                f"rust-code-analysis error on commit {commit.node}, path {path}, function {func['name']}"
+            )
+
+        commit.functions[path].append(
+            {
+                "name": func["name"],
+                "start": func["start_line"],
+                "end": func["end_line"],
+                "metrics": metrics_dict,
+            }
+        )
+
+
+def transform(hg: hglib.client, repo_dir: str, commit: Commit) -> Commit:
     hg_modified_files(hg, commit)
 
     if commit.ignored or len(commit.backsout) > 0 or commit.bug_id is None:
@@ -610,25 +666,41 @@ def transform(hg: hglib.client, repo_dir: str, commit: Commit):
                 source_code_sizes.append(size)
 
                 if type_ != "IDL/IPDL/WebIDL":
-                    metrics = code_analysis_server.metrics(path, after, unit=False)
-                    if metrics.get("spaces"):
+                    after_metrics = code_analysis_server.metrics(
+                        path, after, unit=False
+                    )
+                    if after_metrics.get("spaces"):
                         metrics_file_count += 1
-                        error = get_metrics(commit, metrics["spaces"])
-                        if error:
-                            logger.debug(
-                                f"rust-code-analysis error on commit {commit.node}, path {path}"
-                            )
 
-                        touched_functions = get_touched_functions(
-                            metrics["spaces"],
+                        before_metrics = {}
+                        if not stats["new"]:
+                            try:
+                                before = hg.cat(
+                                    [os.path.join(repo_dir, path).encode("utf-8")],
+                                    rev=commit.node.encode("ascii"),
+                                )
+
+                                before_metrics = code_analysis_server.metrics(
+                                    path, before, unit=False
+                                )
+                            except hglib.error.CommandError as e:
+                                if b"no such file in rev" not in e.err:
+                                    raise
+
+                        set_commit_metrics(
+                            commit,
+                            path,
                             stats["deleted_lines"],
                             stats["added_lines"],
+                            before_metrics,
+                            after_metrics,
                         )
-                        if len(touched_functions) > 0:
-                            commit.functions[path] = list(touched_functions)
 
                     # Replace type with "Objective-C/C++" if rust-code-analysis detected this is an Objective-C/C++ file.
-                    if type_ == "C/C++" and metrics.get("language") == "obj-c/c++":
+                    if (
+                        type_ == "C/C++"
+                        and after_metrics.get("language") == "obj-c/c++"
+                    ):
                         type_ = "Objective-C/C++"
 
             commit.types.add(type_)
@@ -668,33 +740,15 @@ def transform(hg: hglib.client, repo_dir: str, commit: Commit):
     commit.minimum_test_file_size = min(test_sizes, default=0)
 
     if metrics_file_count:
-        commit.average_cyclomatic = commit.total_cyclomatic / metrics_file_count
-        commit.average_halstead_n2 = commit.total_halstead_n2 / metrics_file_count
-        commit.average_halstead_N2 = commit.total_halstead_N2 / metrics_file_count
-        commit.average_halstead_n1 = commit.total_halstead_n1 / metrics_file_count
-        commit.average_halstead_N1 = commit.total_halstead_N1 / metrics_file_count
-        commit.average_source_loc = commit.total_source_loc / metrics_file_count
-        commit.average_instruction_loc = (
-            commit.total_instruction_loc / metrics_file_count
-        )
-        commit.average_logical_loc = commit.total_logical_loc / metrics_file_count
-        commit.average_comment_loc = commit.total_comment_loc / metrics_file_count
-        commit.average_nargs = commit.total_nargs / metrics_file_count
-        commit.average_nexits = commit.total_nexits / metrics_file_count
+        for metric in METRIC_NAMES:
+            commit.metrics[f"{metric}_avg"] = (
+                commit.metrics[f"{metric}_total"] / metrics_file_count
+            )
     else:
         # these values are initialized with sys.maxsize (because we take the min)
         # if no files, then reset them to 0 (it'd be stupid to have min > max)
-        commit.minimum_cyclomatic = 0
-        commit.minimum_halstead_N2 = 0
-        commit.minimum_halstead_n2 = 0
-        commit.minimum_halstead_N1 = 0
-        commit.minimum_halstead_n1 = 0
-        commit.minimum_source_loc = 0
-        commit.minimum_instruction_loc = 0
-        commit.minimum_logical_loc = 0
-        commit.minimum_comment_loc = 0
-        commit.minimum_nargs = 0
-        commit.minimum_nexits = 0
+        for metric in METRIC_NAMES:
+            commit.metrics[f"{metric}_min"] = 0
 
     return commit
 
@@ -732,7 +786,7 @@ def hg_log(hg: hglib.client, revs: List[bytes]) -> Tuple[Commit, ...]:
 
         reviewers = (
             list(set(sys.intern(r) for r in rev[7].decode("utf-8").split(" ")))
-            if rev[7] != b""
+            if rev[7] not in (b"", b"testonly", b"gaia-bump", b"me")
             else []
         )
 
@@ -1149,6 +1203,14 @@ def download_commits(
         if revs is None:
             revs = get_revs(hg, rev_start)
 
+        if save or not os.path.exists("data/component_mapping.lmdb"):
+            logger.info("Downloading file->component mapping...")
+            download_component_mapping()
+
+        if save or not os.path.exists("data/coverage_mapping.lmdb"):
+            logger.info("Downloading commit->coverage mapping...")
+            download_coverage_mapping()
+
         if len(revs) == 0:
             logger.info("No commits to analyze")
             return tuple()
@@ -1163,14 +1225,6 @@ def download_commits(
         else:
             commits = hg_log(hg, revs)
 
-        if save or not os.path.exists("data/component_mapping.lmdb"):
-            logger.info("Downloading file->component mapping...")
-            download_component_mapping()
-
-        if save or not os.path.exists("data/coverage_mapping.lmdb"):
-            logger.info("Downloading commit->coverage mapping...")
-            download_coverage_mapping()
-
         set_commits_to_ignore(hg, repo_dir, commits)
 
         commits_num = len(commits)
@@ -1178,9 +1232,10 @@ def download_commits(
         logger.info(f"Mining {commits_num} patches...")
 
         global code_analysis_server
-        code_analysis_server = rust_code_analysis_server.RustCodeAnalysisServer()
 
         if not use_single_process:
+            code_analysis_server = rust_code_analysis_server.RustCodeAnalysisServer()
+
             with concurrent.futures.ProcessPoolExecutor(
                 initializer=_init_process, initargs=(repo_dir,)
             ) as executor:
@@ -1188,6 +1243,8 @@ def download_commits(
                 commits_iter = tqdm(commits_iter, total=commits_num)
                 commits = tuple(commits_iter)
         else:
+            code_analysis_server = rust_code_analysis_server.RustCodeAnalysisServer(1)
+
             get_component_mapping()
 
             commits = tuple(transform(hg, repo_dir, c) for c in commits)
@@ -1218,6 +1275,10 @@ def download_commits(
 
 
 def update_commits() -> None:
+    if not os.path.exists("data/coverage_mapping.lmdb"):
+        logger.info("Downloading commit->coverage mapping...")
+        download_coverage_mapping()
+
     commits = list(
         get_commits(include_no_bug=True, include_backouts=True, include_ignored=True)
     )
