@@ -9,7 +9,7 @@ import requests
 
 from bugbug import bugzilla, db
 from bugbug.models import get_model_class
-from bugbug.utils import download_check_etag, zstd_decompress
+from bugbug.utils import download_model
 
 MODELS_WITH_TYPE = ("component",)
 
@@ -17,7 +17,7 @@ basicConfig(level=INFO)
 logger = getLogger(__name__)
 
 
-def classify_bugs(model_name, classifier, bug_id):
+def classify_bugs(model_name: str, classifier: str, bug_id: int) -> None:
     if classifier != "default":
         assert (
             model_name in MODELS_WITH_TYPE
@@ -31,18 +31,13 @@ def classify_bugs(model_name, classifier, bug_id):
     if not os.path.exists(model_file_name):
         logger.info(f"{model_file_name} does not exist. Downloading the model....")
         try:
-            download_check_etag(
-                f"https://community-tc.services.mozilla.com/api/index/v1/task/project.relman.bugbug.train_{model_name}.latest/artifacts/public/{model_file_name}.zst"
-            )
+            download_model(model_name)
         except requests.HTTPError:
             logger.error(
                 "A pre-trained model is not available, you will need to train it yourself using the trainer script"
             )
             raise SystemExit(1)
 
-        zstd_decompress(model_file_name)
-        assert os.path.exists(model_file_name), "Decompressed file doesn't exist"
-
     model_class = get_model_class(model_name)
     model = model_class.load(model_file_name)
 
@@ -79,7 +74,7 @@ def classify_bugs(model_name, classifier, bug_id):
         input()
 
 
-def main():
+def main() -> None:
     description = "Perform evaluation on bugs using the specified model"
     parser = argparse.ArgumentParser(description=description)
 
@@ -90,7 +85,7 @@ def main():
         choices=["default", "nn"],
         default="default",
     )
-    parser.add_argument("--bug-id", help="Classify the given bug id")
+    parser.add_argument("--bug-id", help="Classify the given bug id", type=int)
 
     args = parser.parse_args()
 
